import {Vector2} from '../modules/vectors.js';
import {Time} from '../modules/utility.js';
import {Circle} from '../modules/graphicObjects.js';

const time = new Time();

const canv = document.getElementById('fullCanvas');
let c, center;
function getBounds(){
  canv.width = parseFloat(getComputedStyle(canv).width); 
  canv.height =  parseFloat(getComputedStyle(canv).height);
  c = canv.getContext('2d');
  center = new Vector2(canv.width/2, canv.height/2);
}
getBounds();
window.addEventListener('resize', getBounds);
//End of setup

//Get the variables from the right handle
let randomness = document.getElementById('randomness');
let spread = document.getElementById('spread');
let speed = document.getElementById('speed');
let Xaxis = document.getElementById('X-axis');
let Yaxis = document.getElementById('Y-axis');



//Generate the random seeds
let randomSeeds;
document.generateRandomSeed = ()=>{
  randomSeeds = {spread: [], speed: []};
  for (var i = 0; i<150; i++){
    randomSeeds['spread'].push(Math.random() * 50);
    randomSeeds['speed'].push(Math.random() * 0.004);
  }
}
document.generateRandomSeed();


//Function to generate the noise
function AddChaosLayers(offset){
  let output = 0, randomOffset;
  
  for (var i = 0; i < randomness.value; i++){
    randomOffset = (i + offset) % randomSeeds['speed'].length;
    let spreadness = randomSeeds['spread'][randomOffset] * spread.value / Math.log(i+2);//((i+1) * 2);
    let innerValue = time.totalTime * randomSeeds['speed'][randomOffset] * speed.value;
    if (parseInt(randomSeeds['speed'][randomOffset]) % 2 == 0){
      output += Math.sin(innerValue) * spreadness;      
    } else {
      output += Math.cos(innerValue) * spreadness;      
    }
  }
  return output;
}

//Create the update function
let object = new Circle(center.x, center.y, 30);
let updateObj = ()=>{
  object.x = center.x + AddChaosLayers(0) * Xaxis.checked;
  object.y = center.y + AddChaosLayers(5) * Yaxis.checked;
  object.draw(c);
};

function update(){
  c.clearRect(0, 0, canv.width, canv.height);
  
  c.strokeStyle = 'crimson';
  c.beginPath();
  c.moveTo(center.x, 0);
  c.lineTo(center.x, canv.height);
  c.stroke();

  c.beginPath();
  c.moveTo(0, center.y);
  c.lineTo(canv.width, center.y);
  c.stroke();
  
  updateObj();
    
  time.update();
}
setInterval(update, 20);

//Exports random Values to make a repetitive pattern
function getMovementEquation(offset) {
  let text = '';
  let randomOffset;
  
  let moduleAdders = [];
  for (var i = 0; i < randomness.value; i++){
    randomOffset = (i + offset) % randomSeeds['speed'].length;
    let spreadness = randomSeeds['spread'][randomOffset] * spread.value / Math.log(i+2);//((i+1) * 2);
    let innerValue = randomSeeds['speed'][randomOffset] * speed.value;
    if (parseInt(randomSeeds['speed'][randomOffset]) % 2 == 0){
      moduleAdders.push(`sin(time * ${innerValue}) * ${spreadness}`);      
    } else {
      moduleAdders.push(`cos(time * ${innerValue}) * ${spreadness}`);      
    }
  }
  return moduleAdders.join(' + ');
};

document.getMovementEquation = ()=>{
  console.log(`MOVEMENT EQUATION:
  x = ${getMovementEquation(0)};
  y = ${getMovementEquation(5)};`);
};