//CREATE UpdateValues();
//IMPLEMENT ORBITATION
//Center ball not updating

import {Vector2} from '../modules/vectors.js';
import {Time, clamp, getRandomColor, GravityConst} from '../modules/utility.js';
import {Circle} from '../modules/graphicObjects.js';

Circle.prototype.draw = (obj)=>{
  c.fillStyle = obj.style;
  c.beginPath();
  c.arc(center.x + obj.x, center.y + obj.y, obj.size, 0, Math.PI*2, false);
  c.fill();
};

const gravityScaleFactor = 1e9;
Circle.prototype.update = (obj)=>{
console.log(obj);
  //F = G * m1 * m2 / R**2
  let Force =(GravityConst * obj.mass * gravityScaleFactor * centralMass.mass) / obj.position.distance(centralMass.position)**2;
  let velMagnitude = Force / obj.size; // a = F/m (* scaleFactor)
  let forceDirection = centralMass.position.substract(obj.position).normalized;
  
  obj.velocity = obj.velocity.sum(new Vector2(velMagnitude * forceDirection.x, velMagnitude * forceDirection.y));
  obj.position = obj.position.sum(obj.velocity);
};

const time = new Time();

let simController = document.getElementById('Simulation state');
let inputDiv =  document.getElementById('inputsDiv');
let centralMassTB = document.getElementById('Central mass');
let orbitalObjectsTB = [];

//late Setup 
const canv = document.getElementById('fullCanvas');
let c, center;
function getBounds(){
  canv.width = parseFloat(getComputedStyle(canv).width); 
  canv.height =  parseFloat(getComputedStyle(canv).height);
  c = canv.getContext('2d');
  center = new Vector2(canv.width/2, canv.height/2);
}
getBounds();
window.addEventListener('resize', getBounds);
//End of setup

//Create objects
let centralMass = new Circle(0, 0, centralMassTB.value / 1000, '#333');
let orbitals = [];
document.UpdateValues = ()=>{
  centralMass.size = clamp(centralMassTB.value / 1000, 1000, 50);
  centralMass.mass = centralMassTB.value;
  for (var i = 0; i<orbitalObjectsTB.length; i++){
    if (i<extraInputCount) orbitals[i].size = clamp(orbitalObjectsTB[i].value / 40, 10, 30);
    else orbitals[i].size = 0;
    orbitals[i].mass = orbitalObjectsTB[i].value;
  }
};


//INPUT MANAGEMENT
let extraInputCount = 0, removedInputs = 0;
document.addInput = ()=>{
  if (extraInputCount + removedInputs != orbitalObjectsTB.length) {
    orbitalObjectsTB[extraInputCount - removedInputs--].parentElement.style.display = 'block';
    return;
  }
  
  let newInput = document.createElement('input');
  const args = {type:`number`, className:"largeInput", id:`ExtraInput${++extraInputCount}`, value:"100", step:"10", onInput: `UpdateValues();`};
  for (var i in args){
    newInput[i] = args[i];
  }
    
  let newEl = document.createElement('div');
  newEl.className = 'variableItem';
  newEl.innerText = `mass ${extraInputCount} `;
  let newColor = getRandomColor();
  newEl.style.color = newColor;
  newEl.style.textShadow = `1px 0 0 #fff, -1px 0 0 #fff, 0 1px 0 #fff, 0 -1px 0 #fff`;
  newEl.style.fontWeight = `bold`;
  newEl.appendChild(newInput);
  document.getElementById('newInputHandler').appendChild(newEl);
  
  orbitalObjectsTB.push(document.getElementById(`ExtraInput${extraInputCount}`));
  orbitals.push(new Circle((Math.random()-0.5)*center.x, (Math.random()-0.5)*center.y, orbitalObjectsTB[extraInputCount-1], newColor, Vector2.random.multiply(Math.random()*10).sum(new Vector2(3, 3))));
  document.UpdateValues();
};
document.addInput();

document.removeInput = ()=>{
  if (removedInputs >= extraInputCount) {removedInputs = extraInputCount; return;}
  
  orbitalObjectsTB[extraInputCount - ++removedInputs].parentElement.style.display = 'none';
  document.UpdateValues();
};

let simState = 'edit';
document.ChangeSimulationMode = () =>{
  if (simState == 'edit') simState = simController.value = "running";
  else simState = simController.value = "edit";
};

//manual control
let movingObject = null;
canv.addEventListener('mousedown', (e)=>{
  movingObject = null;
  let mousePos = new Vector2(e.layerX - center.x,e.layerY -  center.y), lastDist = 100;
  
  orbitals.forEach((el)=>{
    let dist = mousePos.distance(el.position);
    if (dist < el.size && dist < lastDist){
      movingObject = el;
    }
  });
});
canv.addEventListener('mouseup', (e)=>{
  movingObject = null;
});
canv.addEventListener('mousemove', (e)=>{
  if (!movingObject) return;
  
  movingObject.position = movingObject.position.sum(new Vector2(e.movementX, e.movementY));
});

function update(){
  c.clearRect(0, 0, canv.width, canv.height);
  
  draw();
}

function draw(){
  centralMass.draw(centralMass);
  
  for (var i = 0; i<extraInputCount - removedInputs; i++){
    if (simState == 'running') orbitals[i].update(orbitals[i]);
    orbitals[i].draw(orbitals[i]);
  }
    //console.log(orbitals[0].position);
  //centralMass.draw(centralMass);
}

setInterval(update, 10);